@@ -442,7 +442,8 @@ void Document::Save () const
 		const_cast <Document *> (this)->SetReadOnly (false);
 		const_cast <Document *> (this)->SetDirty (false);
 		const_cast <Document *> (this)->m_LastStackSize = m_UndoList.size ();
-		const_cast <Document *> (this)->m_OpID = m_UndoList.front ()->GetID ();
+		if (m_UndoList.size () > 0)
+			const_cast <Document *> (this)->m_OpID = m_UndoList.front ()->GetID ();
 	}
 	catch (int num) {
 		if (xml)
@@ -1205,20 +1206,16 @@ void Document::OnThemeNamesChanged ()
 double Document::GetMedianBondLength ()
 {
 	vector <double> lengths;
-	int max = 128, nb = 0;
-	lengths.reserve (max);
+	int failed;
+	lengths.reserve (128);
 	double result = 0.;
 	stack<map< string, Object * >::iterator> iters;
 	map< string, Object * >::iterator m;
 	Object *Cur = this, *Ob = GetFirstChild (m);
 	while (Ob) {
-		if (Ob->GetType () == gcu::BondType) {
-			if (nb == max) {
-				max += 128;
-				lengths.resize (max);
-			}
-			lengths[nb++] = static_cast <Bond *> (Ob)->Get2DLength ();
-		} else if (Ob->HasChildren ()) {
+		if (Ob->GetType () == gcu::BondType)
+			lengths.push_back (static_cast <Bond *> (Ob)->Get2DLength ());
+		else if (Ob->HasChildren ()) {
 			Cur = Ob;
 			iters.push (m);
 			Ob = Cur->GetFirstChild (m);
@@ -1232,9 +1229,9 @@ double Document::GetMedianBondLength ()
 			Ob = Cur->GetNextChild (m);
 		}
 	}
-	if (nb > 0)
-		go_range_median_inter_nonconst (&lengths[0], nb, &result);
-	return result;
+	if (lengths.size () > 0)
+		failed = go_range_median_inter_nonconst (&lengths[0], lengths.size (), &result);
+	return (failed)? 0: result;
 }
 
 bool Document::SetProperty (unsigned property, char const *value)
@@ -1273,6 +1270,14 @@ bool Document::SetProperty (unsigned property, char const *value)
 		gcu::Document::SetScale (m_Theme->GetBondLength () / length);
 		break;
 	}
+	case GCU_PROP_THEME_SCALE: {
+		char *end;
+		double length = strtod (value, &end);
+		if (*end != 0)
+			return false;
+		gcu::Document::SetScale (1. / length);
+		break;
+	}
 	}
 	return true;
 }
