@@ -103,9 +103,14 @@ The date of the last file modification.
 	GCU_PROP_DOC_MODIFICATION_TIME,
 	// Theme related properties (might be doc properties in some formats)
 /*!
-The default bond length.
+The default bond length, to use when importing files with an unknown theme.
 */
 	GCU_PROP_THEME_BOND_LENGTH,
+/*!
+The number of coordinates units in one real unit. This is to import files with
+a known theme but a scaled unit (like CDX).
+*/
+	GCU_PROP_THEME_SCALE,
 	// Atom properties
 /*!
 The symbol of an atom.
@@ -180,6 +185,22 @@ The Id of the object representing the products.
 */
 	GCU_PROP_ARROW_END_ID,
 /*!
+The Id of the object representing the mesomer.
+*/
+	GCU_PROP_MESOMER,
+/*!
+The Id of the stoichimetry coefficient.
+*/
+	GCU_PROP_STOICHIOMETRY,
+/*!
+The Id of a molecule or of the text representing the molecule
+*/
+	GCU_PROP_MOLECULE,
+/*!
+The Id of a molecule or text attached to an arrow
+*/
+	GCU_PROP_ARROW_OBJECT,
+/*!
 The type of a reaction arrow ("double" for a reversible arrow).
 */
 	GCU_PROP_REACTION_ARROW_TYPE,
